# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-11-14 13:50
from __future__ import unicode_literals

from django.db import migrations
import phonenumber_field.modelfields


class Migration(migrations.Migration):

    dependencies = [
        ('otp_sms', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='smsdevice',
            name='number',
            field=phonenumber_field.modelfields.PhoneNumberField(help_text='The mobile number to deliver tokens to.', max_length=128, verbose_name='Phone number'),
        ),
    ]
